import os
import sys
import socket
import fcntl
import struct
import time
from pprint import pprint
from ifaceinfotools import IfaceInfoTools

"""
this class operate using the files bellow
    - /sys/class/net/*/
    - /proc/net/route
    - /proc/net/tcp
    - /proc/net/udp
"""

class InterfacesInfos(IfaceInfoTools):
    def __init__(self):
        """
        data initialisation
        maybe in the future somme other data will be loaded directly to improve performance and data usability
        at this time the scan of /sys/class/net are performed in the initialisation of the class
        """
        IfaceInfoTools
        self.version = '0.1.7'
        self.github_url = 'https://github.com/atam84/ifaceinfo'
        self.__timestamp = self.timestamp = int(time.time())
        self.__ifaces_info = self.__get_ifaces_info('/sys/class/net')
        self.__ifaces_routes = self.__get_routes('/proc/net/route')
        self.__ifaces_infos__ = self.__set_as_dict(self.ifaces_info())

    ####################
    # Private Methodes
    ####################
    def __set_timestamp(self):
        return int(time.time())

    def __set_as_dict(self, iface):
        """
        Private method that convert array result to dict
        """
        if type(iface) is list:
            if len(iface) == 1:
                return iface[0]
            else:
                _ifaces = {}
                for _iface in iface:
                    _ifaces[_iface['name']] = _iface
                return _ifaces
        elif type(iface) is dict:
            return iface
        return {}

    def __get_timestamp(self):
        """
        get timestamp of data collection
        """
        return self.__timestamp

    def __get_ifaces_info(self, path):
        """
        private methode
        the main function that scan /sys/class/net to collect interfaces informations
        """
        objArray = []
        # check if the path is directory and you have read access """
        if not os.path.isdir(path) or not os.access(path, os.R_OK):
            raise Exception('Error: you are not allowed')
        # let get the elements in this first level directory """
        try:
            netInterfaces = os.listdir(path)
        except Exception as Err:
            raise Exception('Error: Cannot read the main directory - ', Err)
        # let collect network interfaces
        # loop for every network interfaces found
        for iface in netInterfaces:
            # create dict for the new interface
            netIface = {}
            # if network interfaces found in /sys/class/net/ is directory then start the collection of data else do nothing
            if os.path.isdir(os.path.join(path, iface)):
                # create the interface object and begin the data collection
                netIface['name'] = iface        # the name of the interface (directory name found on /sys/class/net/)
                netIface['ip'] = self.ip_address(iface)
                netIface['mask'] = self.network_mask(iface)
                netIface['timestamp'] = self.__get_timestamp()
                # if the ip and mask are not empty then calculate the network adress else return emplty string
                if netIface['ip'] != '' and netIface['mask'] != '':
                    netIface['network_address'] = self.network_address(netIface['ip'], netIface['mask'])
                else:
                    netIface['network_address'] = ''
                # create the path where we go on it /sys/class/net/<iface>/
                netIfaceDir = os.path.join(path, iface)
                # get the interface params
                try:
                    # list the contents of netIfaceDir /sys/class/net/<iface>/
                    netIfaceParams = os.listdir(netIfaceDir)
                except IOError:
                    # if there is and IOError we pass
                    netIfaceParams = []
                    pass
                
                for param in netIfaceParams:
                    # if ower param is a file we collect the data from the file
                    if os.path.isfile(os.path.join(netIfaceDir, param)):
                        # here we read every param file and we put the data collected on the  """
                        try:
                            # python 2 & 3 compatibility
                            # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
                            if sys.version_info.major == 3:
                                fd = open(os.path.join(netIfaceDir, param), 'r', encoding='latin1')
                            else:
                                fd = open(os.path.join(netIfaceDir, param), 'r')
                            paramValue = fd.read()
                            try:
                                fd.close()
                            except Exception:
                                pass
                        except IOError:
                            paramValue = ''
                        if param == 'uevent':
                            netIface[param] = self.__uevent_as_list(paramValue.rstrip())
                        else:
                            netIface[param] = self.__convert_value(paramValue.rstrip())
                     # if ower param is a directory we collect the data under this directory
                    if os.path.isdir(os.path.join(netIfaceDir, param)):
                        # if an lower device detecte ( vdev (iface) connected to another vdev (bridge))
                        if param.startswith('lower_'):
                            # lower position detected many position can be found in the loop
                            # get the device connected name
                            _devname = param.replace('lower_', '')
                            if 'lower' not in netIface:
                                # if lower key doesnt exist in our newly initiated netIface (interface), we create it as dict
                                netIface['lower'] = {}
                            netIface['lower'][_devname] = self.__scan_object(os.path.join(netIfaceDir, param), param)
                        # if an upper device detecte ( vdev (iface) connected to another vdev (bridge))
                        elif param.startswith('upper_'):
                            # upper position detected many position can be found in the loop
                            _devname = param.replace('upper_', '')
                            if 'upper' not in netIface:
                                # if upper key doesnt exist in our newly initiated netIface (interface), we create it as dict
                                netIface['upper'] = {}
                            netIface['upper'][_devname] = self.__scan_object(os.path.join(netIfaceDir, param), param)
                        # if not upper and not lower directory we do it simple :)
                        else:
                            netIface[param] = self.__scan_object(os.path.join(netIfaceDir, param), param)
                objArray.append(netIface)
        return objArray

    def __scan_object(self, path, object_name):
        """
        private methode
        function used to scan the first level of sub directories in /sys/class/net/<iface>/
        """
        scan_obj = {}
        try:
            sub_objects = os.listdir(path)
        except IOError:
            pass
        for sub_object in sub_objects:
            if os.path.isfile(os.path.join(path, sub_object)):
                try:
                    # python 2 & 3 compatibility
                    # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
                    if sys.version_info.major == 3:
                        __fd = open(os.path.join(path, sub_object), 'r', encoding='latin1')
                    else:
                        __fd = open(os.path.join(path, sub_object), 'r')
                    paramValue = __fd.readlines()
                    __fd.close()
                except IOError:
                    paramValue = ''
                ### paramValue = self.read_file_as_table(os.path.join(path, sub_object))
                if len(paramValue) == 1:
                    scan_obj[sub_object] = self.__convert_value(paramValue[0].rstrip())
                else:
                    if sub_object == 'uevent':
                        scan_obj[sub_object] = self.__uevent_as_list('\n'.join(paramValue))
                    else:
                        scan_obj[sub_object] = []
                        for val in paramValue:
                            scan_obj[sub_object].append(self.__convert_value(val.rstrip()))
            #if os.path.isdir(os.path.join(path, sub_object)):
                #scan_obj[sub_object] = __scan_object(os.path.join(path, sub_object), sub_object)
                #print 'Dir: ' + path + '/' + sub_object, sub_object
        return scan_obj

    def __convert_value(self, valuetoconvert):
        """
        private methode that convert from string to int or float or keep string if tye is not detected
        """
        _value = ''
        try:
            _value = int(valuetoconvert)
        except ValueError:
            try:
                _value = float(valuetoconvert)
            except ValueError:
                _value = valuetoconvert
        return _value

    def __get_routes(self, routefile):
        # Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
        _routing = []
        try:
            """
            python 2 & 3 compatibility
            UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
            """
            if sys.version_info.major == 3:
                fd = open(routefile, 'r', encoding='latin1')
            else:
                fd = open(routefile, 'r')
            paramValue = fd.readlines()
        except IOError:
            paramValue = ''
            fd.close()
        if len(paramValue) == 1:
            return {}
        paramValue.pop(0)
        for route in paramValue:
            _route = route.split()
            _routing.append({
                'iface': _route[0],
                'x_dest': _route[1],
                'dest': self.reverse_ip(self.hex2ip(_route[1])),
                'x_gateway': _route[2],
                'gateway': self.reverse_ip(self.hex2ip(_route[2])),
                'flags': _route[3],
                'refcnt': _route[4],
                'use': _route[5],
                'metric': _route[6],
                'x_mask': _route[7],
                'mask': self.reverse_ip(self.hex2ip(_route[7])),
                'mtu': _route[8],
                'window': _route[9],
                'irtt': _route[10],
                'timestamp': self.__get_timestamp()
            })
        return _routing

    def __uevent_as_list(self, uevententry):
        __iface = {}
        __devtype = False
        for iface_type in uevententry.split('\n'):
            _iface_type = iface_type.split('=')
            if len(_iface_type) == 2:
                __iface[_iface_type[0].lower()] = _iface_type[1]
                if _iface_type[0].lower() == 'devtype':
                     __devtype = True
        if __devtype == False:
            __iface['devtype'] = 'unknown'
        return __iface


    ####################
    # Public Methodes
    ####################

    #######################
    # the as dict method
    #######################
    def as_dict(self):
        """
        return interfaces informations as dict result
        """
        return self.__ifaces_infos__

    def get_timestamp(self):
        """
        get timestamp of data collection
        """
        return self.timestamp

    def ifaces_as_dict(self):
        """
        return interfaces informations as dict result give the same result as self.as_dict()
        """
        return self.as_dict()
    
    def ifaces_statistics_as_dict(self):
        """
        return ifaces statistics as dict
        """
        __iface_statistics = {}
        for iface in self.as_dict():
            __iface_statistics[iface] = self.as_dict()[iface]['statistics']
            __iface_statistics[iface]['timetamp'] = self.get_timestamp()
        return __iface_statistics

    def iface_as_dict(self, ifacename):
        """
        return interfaces informations as dict result
        """
        if ifacename in self.as_dict():
            return self.as_dict()[ifacename]

    def list_interfaces(self):
        """
        list all interafces as list()
        """
        return [iface for iface in self.as_dict()]

    ####################
    # initial method
    ####################

    def refresh(self):
        """
        this function refresh the informations collected when the class is loaded
        if you want to work with fresh data use this function to reload the data updated
        """
        self.__init__()
    
    def reload(self):
        """
        same as self.refresh()
        """
        self.refresh()

    def ifaces_count(self):
        """
        return a number of detected interfaces
        """
        return len(self.ifaces_info())

    def ifaces_list(self):
        """
        get a list of interface name
        return array of string
        """
        iface_lst = []
        for iface in self.ifaces_info():
            iface_lst.append(iface['name'])
        return iface_lst

    def iface_info(self, ifaceName):
        """
        get specific interface name with full information
        return dict
        """
        for iface in self.ifaces_info():
            if iface['name'] == ifaceName:
                return iface
        return {}

    def ifaces_info(self):
        """
        get all interfaces informations (full collected information)
        """
        return self.__ifaces_info

    def ifaces_ifindex(self):
        """
        get all interfaces name and ifindex
        return array of dict
        """
        ifacesIndex = []
        for iface in self.ifaces_info():
            _iface = {
                'name': iface['name'],
                'iflink': iface['iflink'],
                'ifindex': iface['ifindex']
            }
            ifacesIndex.append(_iface)
        return ifacesIndex

    def ifaces_iflink(self):
        """
        get all interfaces name and iflink
        return array of dict
        """
        ifacesLink = []
        for iface in self.ifaces_info():
            _iface = {
                'name': iface['name'],
                'iflink': iface['iflink'],
                'ifindex': iface['ifindex']
            }
            ifacesLink.append(_iface)
        return ifacesLink

    def ifaces_ifindex_iflink(self):
        """
        get all interfaces with brief informations (name, ifindex, iflink, ip and mask)
        return an array of dict
        """
        ifaceIndexLink = []
        for iface in self.ifaces_info():
            _iface = {
                'name': iface['name'],
                'ifindex': iface['ifindex'],
                'iflink': iface['iflink'],
                'ip': iface['ip'],
                'mask': iface['mask'],
                'network_address': iface['network_address'],
                'operstate': iface['operstate'],
                'timestamp': self.__get_timestamp()
            }
            ifaceIndexLink.append(_iface)
        return ifaceIndexLink

    def iface_by_uid(self, identifier, value, info='full'):
        """
        get interfaces by defined identifier and value, the third parametre info allow 
        the choise to return the result as a full data or briefly
        
        the identifier can be any key value of the class definition
        this funtion is used by self.getifaceByIndex() and self.getifaceByLink()

        return dict
        """
        _value = str(value)
        for iface in self.ifaces_info():
            try:
                if iface[identifier] == _value:
                    if info == 'brief':
                        return {
                            'name': iface['name'],
                            'ifindex': iface['ifindex'],
                            'iflink': iface['iflink'],
                            'ip': iface['ip'],
                            'mask': iface['mask'],
                            'network_address': iface['network_address'],
                            'operstate': iface['operstate'],
                            'timestamp': self.__get_timestamp()
                        }
                    elif info == 'full':
                        return iface
            except Exception:
                pass
        return {}
    
    def iface_by_ifindex(self, ifindex, info='full'):
        """
        get interface that have ifindex=X
        ifindex is located in /sys/class/net/<iface>/ifindex
        return dict
        """
        return self.iface_by_uid('ifindex', ifindex, info)

    def iface_by_iflink(self, iflink, info='full'):
        """
        get interface that have iflink=X
        ifindex is located in /sys/class/net/<iface>/iflink
        return dict
        """
        return self.iface_by_uid('iflink', iflink, info)

    def ifaces_by_status(self, status, info='full'):
        """
        get interface by status, this function is used by self.getifacesUp() and self.getifacesDown()
        return an array with dict
        """
        _ifaces = []
        for iface in self.ifaces_info():
            try:
                if iface['operstate'] == status:
                    if info == 'brief':
                        _iface = {
                            'name': iface['name'],
                            'ifindex': iface['ifindex'],
                            'iflink': iface['iflink'],
                            'ip': iface['ip'],
                            'mask': iface['mask'],
                            'network_address': iface['network_address'],
                            'operstate': iface['operstate'],
                            'timestamp': self.__get_timestamp()
                        }
                    elif info == 'full':
                        _iface = iface
                    _ifaces.append(_iface)
            except Exception:
                pass
        return _ifaces

    def ifaces_up(self, info='full'):
        """
        get interface up only
        return an array with dict
        """
        return self.ifaces_by_status('up', info)

    def ifaces_down(self, info='full'):
        """
        get interface down and other status
        return an array with dict
        """
        return self.ifaces_by_status('down', info)

    #def get_ip_address(ifname):
    #    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    #    return socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915, struct.pack('256s', ifname[:15]))[20:24])

    def ifaces_type(self):
        """
        return a list of all interfaces with uevent that give device type
        """
        #'uevent': 'DEVTYPE=wlan\nINTERFACE=wlp2s0\nIFINDEX=3'
        _ifacetype = []
        for iface in self.ifaces_info():
            _iface = {
                'name': iface['name'],
                'ip': iface['ip'],
                'mask': iface['mask'],
                'network_address': iface['network_address'],
                'operstate': iface['operstate'],
                'timestamp': self.__get_timestamp(),
                'uevent': iface['uevent']
            }
            _ifacetype.append(_iface)
        return _ifacetype

    def iface_type(self, ifacename):
        """
        return a dict of specific interface with interface_type if the interface is not found return empty dict
        """
        #'uevent': 'DEVTYPE=wlan\nINTERFACE=wlp2s0\nIFINDEX=3'
        for iface in self.ifaces_type():
            if iface['name'] == ifacename:
                return iface
        return {}

    def ifaces_network_config(self):
        """
        get the interfaces network configuration only
        """
        _netconf = []
        for iface in self.ifaces_info():
            _netconf.append({
                'name': iface['name'], 
                'ip': iface['ip'],
                'mask': iface['mask'],
                'network_address': iface['network_address'],
                'mtu': iface['mtu'],
                'link_mode': iface['link_mode'],
                'proto_down': iface['proto_down'],
                'operstate': iface['operstate'],
                'broadcast': iface['broadcast'],
                'speed': iface['speed'],
                'addr_assign_type': iface['addr_assign_type'],
                'type': iface['type'],
                'name_assign_type': iface['name_assign_type'],
                'dev_id': iface['dev_id'],
                'address': iface['address'],
                'timestamp': self.__get_timestamp()
            })
        return _netconf
    
    def iface_network_config(self, ifacename):
        """
        get specific interface network configuration only, return dict
        """
        for iface in self.ifaces_info():
            if iface['name'] == ifacename:
                return {
                    'name': iface['name'], 
                    'ip': iface['ip'],
                    'mask': iface['mask'],
                    'network_address': iface['network_address'],
                    'mtu': iface['mtu'],
                    'link_mode': iface['link_mode'],
                    'proto_down': iface['proto_down'],
                    'operstate': iface['operstate'],
                    'broadcast': iface['broadcast'],
                    'speed': iface['speed'],
                    'addr_assign_type': iface['addr_assign_type'],
                    'type': iface['type'],
                    'name_assign_type': iface['name_assign_type'],
                    'dev_id': iface['dev_id'],
                    'address': iface['address'],
                    'timestamp': self.__get_timestamp()
                }
            else:
                return {}

    def ifaces_statistics(self):
        """
        return a list of all interfaces with statistics
        """
        _ifacestatistics = []
        for iface in self.ifaces_info():
            _ifacestatistics.append({
                'name': iface['name'], 
                'ip': iface['ip'],
                'mask': iface['mask'],
                'network_address': iface['network_address'],
                'mtu': iface['mtu'],
                'ifindex': iface['ifindex'],
                'iflink': iface['iflink'],
                'statistics': iface['statistics'],
                'timestamp': self.__get_timestamp()
            })
        return _ifacestatistics
    
    def iface_statistics(self, ifacename):
        """
        return a dict of specific interface with statistics
        """
        for iface in self.ifaces_statistics():
            if iface['name'] == ifacename:
                return iface
        return {}

    def ifaces_routes(self):
        """
        return the routing table as array.
        """
        return self.__ifaces_routes

    def iface_routes(self, ifacename):
        """
        return the routing table of specific interface.
        """
        _iface_routes = []
        for iface_route in self.ifaces_routes():
            if iface_route['iface'] == 'ifacename':
                _iface_routes.append(iface_route)
        return _iface_routes





import os
import sys
import time
from ifaceinfotools import IfaceInfoTools

class InterfacesRoutes(IfaceInfoTools):
    def __init__(self):
        """
        data initialisation
        maybe in the future somme other data will be loaded directly to improve performance and data usability
        at this time the scan of /sys/class/net are performed in the initialisation of the class
        """
        IfaceInfoTools
        self.version = '0.1.2'
        self.github_url = 'https://github.com/atam84/ifaceinfo'
        self.__timestamp = self.timestamp = int(time.time())
        self.__ifaces_routes = self.__get_routes('/proc/net/route')

    def __get_routes(self, routefile):
        # Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
        _routing = []
        try:
            """
            python 2 & 3 compatibility
            UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
            """
            if sys.version_info.major == 3:
                fd = open(routefile, 'r', encoding='latin1')
            else:
                fd = open(routefile, 'r')
            paramValue = fd.readlines()
        except IOError:
            paramValue = ''
            fd.close()
        if len(paramValue) == 1:
            return {}
        paramValue.pop(0)
        for route in paramValue:
            _route = route.split()
            _routing.append({
                'iface': _route[0],
                'x_dest': _route[1],
                'dest': self.reverse_ip(self.hex2ip(_route[1])),
                'x_gateway': _route[2],
                'gateway': self.reverse_ip(self.hex2ip(_route[2])),
                'flags': _route[3],
                'refcnt': _route[4],
                'use': _route[5],
                'metric': _route[6],
                'x_mask': _route[7],
                'mask': self.reverse_ip(self.hex2ip(_route[7])),
                'mtu': _route[8],
                'window': _route[9],
                'irtt': _route[10],
                'timestamp': self.__timestamp
            })
        return _routing

    def ifaces_routes(self):
        """
        return the routing table as array.
        """
        return self.__ifaces_routes

    def iface_routes(self, ifacename):
        """
        return the routing table of specific interface.
        """
        _iface_routes = []
        for iface_route in self.ifaces_routes():
            if iface_route['iface'] == 'ifacename':
                _iface_routes.append(iface_route)
        return _iface_routes







import os
import sys
import socket
import fcntl
import struct
import time


class FileReader:
    def read_file_as_table(self, filename):
        '''
        private method that read given file completly and return it as list of lines
        '''
        try:
            # python 2 & 3 compatibility
            # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
            if sys.version_info.major == 3:
                fd = open(filename, 'r', encoding='latin1')
            else:
                fd = open(filename, 'r')
            filecontents = fd.readlines()
            try:
                fd.close()
            except Exception:
                pass
        except IOError:
            filecontents = ''
        if type(filecontents) is list:
            filecontents.pop(0)
        return filecontents

class IfaceInfoTools:
    def __init__(self):
        self.IfaceInfoTools_version = '0.0.1'

    def ip_address(self, ifacename):
        '''
        private methode
        get ip address of interface using socket interface
        '''
        try:
            addr = socket.inet_ntoa(fcntl.ioctl(socket.socket(socket.AF_INET, socket.SOCK_DGRAM), 0x8915, struct.pack('256s', ifacename.encode()))[20:24])
        except (IOError, UnicodeDecodeError):
            addr = ''
        return addr

    def network_mask(self, ifacename):
        '''
        private methode
        get net mask address of interface using socket interface
        '''
        try:
            mask = socket.inet_ntoa(fcntl.ioctl(socket.socket(socket.AF_INET, socket.SOCK_DGRAM), 35099, struct.pack('256s', ifacename.encode()))[20:24])
        except (IOError, UnicodeDecodeError):
            mask = ''
        return mask

    def network_address(self, ipaddr, mask):
        '''
        private methode
        used to calculate the network address
        '''
        x_ip = self.ip2hex(ipaddr)
        x_mask = self.ip2hex(mask)
        return self.hex2ip(hex(int(x_ip, 16) & int(x_mask, 16)))

    def ip2hex(self, addr):
        '''
        private methode
        convert string decimal ip address to hexadecimal representation
        return string (as hex representation)
        '''
        addr = addr.split('.')
        if len(addr) == 4:
            return '{:02x}{:02x}{:02x}{:02x}'.format(int(addr[0]), int(addr[1]), int(addr[2]), int(addr[3]))
        else:
            return ''

    def hex2ip(self, x_addr):
        '''
        private methode
        convert hexadecimal representation to string decimal ip address
        return string (as ip address representation)
        '''
        addr = '{:08x}'.format(int(x_addr.replace('0x', ''), 16))
        if len(addr) == 8:
            return '{:d}.{:d}.{:d}.{:d}'.format(int(addr[0:2], 16), int(addr[2:4], 16), int(addr[4:6], 16), int(addr[6:8], 16))
        else:
            return ''

    def reverse_ip(self, ipaddress):
        '''
        Private method that reverse ip address
        '''
        _ipaddr = ipaddress.split('.')
        return '{}.{}.{}.{}'.format(_ipaddr[3], _ipaddr[2], _ipaddr[1], _ipaddr[0])


class Conn(IfaceInfoTools, FileReader):
    '''
    this class provide a quick interface to read /proc/net/tcp and give the informations about the current connexion to host
    
    https://www.kernel.org/doc/Documentation/networking/proc_net_tcp.txt
    
    /proc/net/tcp   <<-- Example -->>
    sl  local_address rem_address   st   tx_queue         rx_queue     tr      tm->when retrnsmt   uid  timeout inode                                                     
    0: 0100007F:0035 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 32059 1 0000000000000000 100 0 0 10 0                     
    1: 00000000:0016 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 44971 1 0000000000000000 100 0 0 10 0                     
    2: 0100007F:0277 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 24125 1 0000000000000000 100 0 0 10 0                     
    3: 0100007F:6989 00000000:0000 0A 00000000:00000000 00:00000000 00000000   122        0 26080 1 0000000000000000 100 0 0 10 0                     
    4: 00000000:14EB 00000000:0000 0A 00000000:00000000 00:00000000 00000000   102        0 25337 1 0000000000000000 100 0 0 10 0                     
    5: 2401A8C0:AC56 2401A8C0:0016 01 00000000:00000000 02:0007DDFA 00000000  1000        0 59815 3 0000000000000000 20 4 30 10 -1                    
    6: 2401A8C0:0016 2401A8C0:AC56 01 00000000:00000000 02:0007DDFA 00000000     0        0 61538 4 0000000000000000 20 4 31 10 17                    
    7: 2401A8C0:E71E CEC93AD8:01BB 01 00000000:00000000 00:00000000 00000000  1000        0 69532 1 0000000000000000 39 4 30 10 -1                    
    8: 2401A8C0:E360 8A13D9AC:01BB 01 00000000:00000000 00:00000000 00000000  1000        0 72008 1 0000000000000000 158 4 30 10 -1                   
    '''
    def __init__(self, procfilename, protocole):
        '''
        initialization method that inherit IfaceInfoTools class
        '''
        FileReader
        IfaceInfoTools
        self.__state = {
            '01': 'established',
            '02': 'syn_sent',
            '03': 'syn_recv',
            '04': 'fin_wait1',
            '05': 'fin_wait2',
            '06': 'time_wait',
            '07': 'close',
            '08': 'close_wait',
            '09': 'last_ack',
            '0A': 'listen',
            '0B': 'closing',
            '0C': 'max_states'
        }
        self.__timestamp = int(time.time())
        self.__connexion = self.__connexion_parser(procfilename, protocole)
        #self.tcp6_connexion = self.__tcp6_connexion_parser('/proc/net/tcp6')

    def __code_state(self, hexcode):
        '''
        private method that return the connexion status from the code of the connexion status (st)
        '''
        if hexcode in self.__state:
            return self.__state[hexcode]
        return 'unknwon'

    def __connexion_parser(self, filename, protocle):
        '''
        private function that parse the /proc/net/{tcp, udp} and give human readable ip, ports and status ...
        '''
        _connexion = []
        _jump = 4
        for _tcpconn in self.read_file_as_table(filename):
            _element = _tcpconn.split()
            _localaddr = _element[1].split(':')
            _remotraddr = _element[2].split(':')
            if protocle == 'tcp' or protocle == 'udp':
                _ip_local = self.reverse_ip(self.hex2ip(_localaddr[0]))
                _ip_remote = self.reverse_ip(self.hex2ip(_remotraddr[0]))
            elif protocle == 'tcp6' or protocle == 'udp6':
                _ip_local = ':'.join([_localaddr[0][i:i+_jump] for i in range(0, len(_localaddr[0]), _jump)])
                _ip_remote = ':'.join([_remotraddr[0][i:i+_jump] for i in range(0, len(_remotraddr[0]), _jump)])
            _connexion.append({
                'sl': _element[0].replace(':', ''),
                'local_address': {
                    'x_addr': _localaddr[0],
                    'x_port': _localaddr[1],
                    'ip': _ip_local,
                    'port': int(_localaddr[1], 16)
                },
                'rem_address': {
                    'x_addr': _remotraddr[0],
                    'x_port': _remotraddr[1],
                    'ip': _ip_remote,
                    'port': int(_remotraddr[1], 16)
                },
                'st': _element[3],
                'tx_queue': _element[4],
                'rx_queue': _element[5],
                'tr': _element[6],
                'tm->when': _element[7],
                'retrnsmt': _element[8],
                'uid': _element[9],
                'timeout': _element[10],
                'inode': _element[11],
                'protocole': protocle,
                'status': self.__code_state(_element[3])
            })
        return _connexion

    def connexion(self):
        '''
        return a list of all {tcp, udp} connexion
        '''
        return self.__connexion

    def conn(self):
        '''
        (alias) same is connexion(self)
        '''
        return self.connexion()

    def conn_by_status(self, connstatus):
        '''
        get {tcp, udp} connexion with last_ack '09' {tcp, udp} status
        '''
        _connstatus = connstatus.replace('0x', '')
        __statusconn = []
        for __conn in self.connexion():
            if __conn['st'] == _connstatus or __conn['status'] == _connstatus:
                __statusconn.append(__conn)
        return __statusconn

    def get_established_conn(self):
        '''
        get {tcp, udp} connexion with established '01' {tcp, udp} status
        '''
        return self.conn_by_status('0x01')

    def get_syn_sent_conn(self):
        '''
        get {tcp, udp} connexion with syn_sent '02' {tcp, udp} status
        '''
        return self.conn_by_status('0x02')

    def get_syn_recv_conn(self):
        '''
        get {tcp, udp} connexion with recv '03' {tcp, udp} status
        '''
        return self.conn_by_status('0x03')

    def get_fin_wait1_conn(self):
        '''
        get {tcp, udp} connexion with fin_wait1 '04' {tcp, udp} status
        '''
        return self.conn_by_status('0x04')

    def get_fin_wait2_conn(self):
        '''
        get {tcp, udp} connexion with fin_wait2 '05' {tcp, udp} status
        '''
        return self.conn_by_status('0x05')

    def get_time_wait_conn(self):
        '''
        get {tcp, udp} connexion with time_wait '06' {tcp, udp} status
        '''
        return self.conn_by_status('0x06')

    def get_close_conn(self):
        '''
        get {tcp, udp} connexion with close '07' {tcp, udp} status
        '''
        return self.conn_by_status('0x07')

    def get_close_wait_conn(self):
        return self.conn_by_status('0x08')

    def get_last_ack_conn(self):
        '''
        get {tcp, udp} connexion with last_ack '09' {tcp, udp} status
        '''
        return self.conn_by_status('0x09')

    def get_listen_conn(self):
        '''
        get {tcp, udp} connexion with listen '0A' {tcp, udp} status
        '''
        return self.conn_by_status('0x0A')

    def get_closing_conn(self):
        '''
        get {tcp, udp} connexion with closing '0B' {tcp, udp} status
        '''
        return self.conn_by_status('0x0B')

    def get_max_state_conn(self):
        '''
        get {tcp, udp} connexion with max_state '0C' {tcp, udp} status
        '''
        return self.conn_by_status('0x0C')

    def __get_conn_data(self, what='port', scope='local', value=0):
        '''
        this private function is not used
        '''
        _conn = []
        if scope == 'local':
            _scope = 'local_address'
        elif scope == 'remote':
            _scope = 'rem_address'
        elif scope == 'none':
            _scope = 'none'
        if _scope == 'none':
            for _conn in self.connexion():
                if _conn[what] == value:
                    _conn.append(_conn)
        else:
            for _conn in self.connexion():
                if _conn[scope][what] == value:
                    _conn.append(_conn)
        return _conn

    def remote_ports(self):
        '''
        get list of ports used by remote host
        '''
        _ports = []
        for _conn in self.connexion():
            _ports.append(_conn['rem_address']['port'])
        return list(set(_ports))

    def local_ports(self):
        '''
        get list of ports used by local host [listen, established, ...]
        '''
        _ports = []
        for _conn in self.connexion():
            _ports.append(_conn['local_address']['port'])
        return list(set(_ports))

    def remote_addrs(self):
        '''
        get dict of remote address and ports list
        {
            'remote_host_ip': [list of ports],
        }
        '''
        _addrs = {}
        for _conn in self.connexion():
            if _conn['rem_address']['ip'] not in _addrs:
                _addrs[_conn['rem_address']['ip']] = []
            if _conn['rem_address']['port'] not in _addrs[_conn['rem_address']['ip']]:
                _addrs[_conn['rem_address']['ip']].append(_conn['rem_address']['port'])
        return _addrs

    def local_addrs(self):
        '''
        get dict of local address and ports list
        {
            'local_host_ip': [list of ports]
        }
        '''
        _addrs = {}
        for _conn in self.connexion():
            if _conn['local_address']['ip'] not in _addrs:
                _addrs[_conn['local_address']['ip']] = []
            if _conn['local_address']['port'] not in _addrs[_conn['local_address']['ip']]:
                _addrs[_conn['local_address']['ip']].append(_conn['local_address']['port'])
        return _addrs





import os
import sys
#import socket
#import fcntl
#import struct
import time
from pprint import pprint
from ifaceinfotools import Conn

class UDPConn(Conn):
    def __init__(self):
        Conn.__init__(self, '/proc/net/udp', 'udp')

    def refresh(self):
        '''
        this function refresh the informations collected when the class is loaded
        if you want to work with fresh data use this function to reload the data updated
        '''
        self.__init__()
    
    def reload(self):
        '''
        same as self.refresh()
        '''
        self.refresh()

class UDP6Conn(Conn):
    def __init__(self):
        Conn.__init__(self, '/proc/net/udp6', 'udp6')

    def refresh(self):
        '''
        this function refresh the informations collected when the class is loaded
        if you want to work with fresh data use this function to reload the data updated
        '''
        self.__init__()
    
    def reload(self):
        '''
        same as self.refresh()
        '''
        self.refresh()


from .ifaceinfotools import FileReader
from .ifaceinfotools import IfaceInfoTools
from .ifaceversion import version
from .ifaceinfotools import Conn
from .tcpconn import TCPConn
from .tcpconn import TCP6Conn
from .udpconn import UDPConn
from .udpconn import UDP6Conn
from .ifaceroutes import InterfacesRoutes
from .ifaceinfo import InterfacesInfos



import os
import sys
#import socket
#import fcntl
#import struct
import time
from pprint import pprint
from ifaceinfotools import Conn

class TCPConn(Conn):
    def __init__(self):
        Conn.__init__(self, '/proc/net/tcp', 'tcp')

    def refresh(self):
        '''
        this function refresh the informations collected when the class is loaded
        if you want to work with fresh data use this function to reload the data updated
        '''
        self.__init__()
    
    def reload(self):
        '''
        same as self.refresh()
        '''
        self.refresh()

class TCP6Conn(Conn):
    def __init__(self):
        Conn.__init__(self, '/proc/net/tcp6', 'tcp6')

    def refresh(self):
        '''
        this function refresh the informations collected when the class is loaded
        if you want to work with fresh data use this function to reload the data updated
        '''
        self.__init__()
    
    def reload(self):
        '''
        same as self.refresh()
        '''
        self.refresh()

#class TCPConn(IfaceInfoTools):
#    '''
#    this class provide a quick interface to read /proc/net/tcp and give the informations about the current connexion to host
#    
#    https://www.kernel.org/doc/Documentation/networking/proc_net_tcp.txt
#    
#    /proc/net/tcp   <<-- Example -->>
#    sl  local_address rem_address   st   tx_queue         rx_queue     tr      tm->when retrnsmt   uid  timeout inode                                                     
#    0: 0100007F:0035 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 32059 1 0000000000000000 100 0 0 10 0                     
#    1: 00000000:0016 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 44971 1 0000000000000000 100 0 0 10 0                     
#    2: 0100007F:0277 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 24125 1 0000000000000000 100 0 0 10 0                     
#    3: 0100007F:6989 00000000:0000 0A 00000000:00000000 00:00000000 00000000   122        0 26080 1 0000000000000000 100 0 0 10 0                     
#    4: 00000000:14EB 00000000:0000 0A 00000000:00000000 00:00000000 00000000   102        0 25337 1 0000000000000000 100 0 0 10 0                     
#    5: 2401A8C0:AC56 2401A8C0:0016 01 00000000:00000000 02:0007DDFA 00000000  1000        0 59815 3 0000000000000000 20 4 30 10 -1                    
#    6: 2401A8C0:0016 2401A8C0:AC56 01 00000000:00000000 02:0007DDFA 00000000     0        0 61538 4 0000000000000000 20 4 31 10 17                    
#    7: 2401A8C0:E71E CEC93AD8:01BB 01 00000000:00000000 00:00000000 00000000  1000        0 69532 1 0000000000000000 39 4 30 10 -1                    
#    8: 2401A8C0:E360 8A13D9AC:01BB 01 00000000:00000000 00:00000000 00000000  1000        0 72008 1 0000000000000000 158 4 30 10 -1                   
#    '''
#    def __init__(self):
#        '''
#        initialization method that inherit IfaceInfoTools class
#        '''
#        IfaceInfoTools
#        self.__tcpstate = {
#            '01': 'established',
#            '02': 'syn_sent',
#            '03': 'syn_recv',
#            '04': 'fin_wait1',
#            '05': 'fin_wait2',
#            '06': 'time_wait',
#            '07': 'close',
#            '08': 'close_wait',
#            '09': 'last_ack',
#            '0A': 'listen',
#            '0B': 'closing',
#            '0C': 'max_states'
#        }
#        self.__timestamp = int(time.time())
#        self.__tcp_connexion = self.__tcp_connexion_parser('/proc/net/tcp', 'tcp')
#        #self.tcp6_connexion = self.__tcp6_connexion_parser('/proc/net/tcp6')
#
#    def __code_tcpstate(self, hexcode):
#        '''
#        private method that return the connexion status from the code of the connexion status (st)
#        '''
#        if hexcode in self.__tcpstate:
#            return self.__tcpstate[hexcode]
#        return 'unknwon'
#
#    def __read_file_as_table(self, filename):
#        '''
#        private method that read given file completly and return it as list of lines
#        '''
#        try:
#            # python 2 & 3 compatibility
#            # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xe7 in position 2: invalid continuation byte
#            if sys.version_info.major == 3:
#                fd = open(filename, 'r', encoding='latin1')
#            else:
#                fd = open(filename, 'r')
#            filecontents = fd.readlines()
#            try:
#                fd.close()
#            except Exception:
#                pass
#        except IOError:
#            filecontents = False
#        if type(filecontents) is list:
#            filecontents.pop(0)
#        return filecontents
#
#    def __tcp_connexion_parser(self, filename, protocle):
#        '''
#        private function that parse the /proc/net/tcp and give human readable ip, ports and status ...
#        '''
#        _tcpconnexion = []
#        for _tcpconn in self.__read_file_as_table(filename):
#            _element = _tcpconn.split()
#            _localaddr = _element[1].split(':')
#            _remotraddr = _element[2].split(':')
#            _tcpconnexion.append({
#                'sl': _element[0].replace(':', ''),
#                'local_address': {
#                    'x_addr': _localaddr[0],
#                    'x_port': _localaddr[1],
#                    'ip': self.reverse_ip(self.hex2ip(_localaddr[0])),
#                    'port': int(_localaddr[1], 16)
#                },
#                'rem_address': {
#                    'x_addr': _remotraddr[0],
#                    'x_port': _remotraddr[1],
#                    'ip': self.reverse_ip(self.hex2ip(_remotraddr[0])),
#                    'port': int(_remotraddr[1], 16)
#                },
#                'st': _element[3],
#                'tx_queue': _element[4],
#                'rx_queue': _element[5],
#                'tr': _element[6],
#                'tm->when': _element[7],
#                'retrnsmt': _element[8],
#                'uid': _element[9],
#                'timeout': _element[10],
#                'inode': _element[11],
#                'protocole': protocle,
#                'status': self.__code_tcpstate(_element[3])
#            })
#        return _tcpconnexion
#
#    def refresh(self):
#        '''
#        this function refresh the informations collected when the class is loaded
#        if you want to work with fresh data use this function to reload the data updated
#        '''
#        self.__init__()
#    
#    def reload(self):
#        '''
#        same as self.refresh()
#        '''
#        self.refresh()
#
#    def tcp_connexion(self):
#        '''
#        return a list of all tcp connexion
#        '''
#        return self.__tcp_connexion
#
#    def tcp_conn(self):
#        '''
#        (alias) same is tcp_connexion(self)
#        '''
#        return self.tcp_connexion()
#
#    def conn_by_status(self, connstatus):
#        '''
#        get tcp connexion with last_ack '09' tcp status
#        '''
#        _connstatus = connstatus.replace('0x', '')
#        _tcpconn = []
#        for __conn in self.tcp_connexion():
#            if __conn['st'] == _connstatus or __conn['status'] == _connstatus:
#                _tcpconn.append(__conn)
#        return _tcpconn
#
#    def get_established_tcp_conn(self):
#        '''
#        get tcp connexion with established '01' tcp status
#        '''
#        return self.conn_by_status('0x01')
#
#    def get_syn_sent_tcp_conn(self):
#        '''
#        get tcp connexion with syn_sent '02' tcp status
#        '''
#        return self.conn_by_status('0x02')
#
#    def get_syn_recv_tcp_conn(self):
#        '''
#        get tcp connexion with recv '03' tcp status
#        '''
#        return self.conn_by_status('0x03')
#
#    def get_fin_wait1_tcp_conn(self):
#        '''
#        get tcp connexion with fin_wait1 '04' tcp status
#        '''
#        return self.conn_by_status('0x04')
#
#    def get_fin_wait2_tcp_conn(self):
#        '''
#        get tcp connexion with fin_wait2 '05' tcp status
#        '''
#        return self.conn_by_status('0x05')
#
#    def get_time_wait_tcp_conn(self):
#        '''
#        get tcp connexion with time_wait '06' tcp status
#        '''
#        return self.conn_by_status('0x06')
#
#    def get_close_tcp_conn(self):
#        '''
#        get tcp connexion with close '07' tcp status
#        '''
#        return self.conn_by_status('0x07')
#
#    def get_close_wait_tcp_conn(self):
#        return self.conn_by_status('0x08')
#
#    def get_last_ack_tcp_conn(self):
#        '''
#        get tcp connexion with last_ack '09' tcp status
#        '''
#        return self.conn_by_status('0x09')
#
#    def get_listen_tcp_conn(self):
#        '''
#        get tcp connexion with listen '0A' tcp status
#        '''
#        return self.conn_by_status('0x0A')
#
#    def get_closing_tcp_conn(self):
#        '''
#        get tcp connexion with closing '0B' tcp status
#        '''
#        return self.conn_by_status('0x0B')
#
#    def get_max_state_tcp_conn(self):
#        '''
#        get tcp connexion with max_state '0C' tcp status
#        '''
#        return self.conn_by_status('0x0C')
#
#    def __get_conn_data(self, what='port', scope='local', value=0):
#        '''
#        this private function is not used
#        '''
#        _tcpconn = []
#        if scope == 'local':
#            _scope = 'local_address'
#        elif scope == 'remote':
#            _scope = 'rem_address'
#        elif scope == 'none':
#            _scope = 'none'
#        if _scope == 'none':
#            for _conn in self.tcp_connexion():
#                if _conn[what] == value:
#                    _tcpconn.append(_conn)
#        else:
#            for _conn in self.tcp_connexion():
#                if _conn[scope][what] == value:
#                    _tcpconn.append(_conn)
#        return _tcpconn
#
#    def remote_ports(self):
#        '''
#        get list of ports used by remote host
#        '''
#        _ports = []
#        for _conn in self.tcp_connexion():
#            _ports.append(_conn['rem_address']['port'])
#        return list(set(_ports))
#
#    def local_ports(self):
#        '''
#        get list of ports used by local host [listen, established, ...]
#        '''
#        _ports = []
#        for _conn in self.tcp_connexion():
#            _ports.append(_conn['local_address']['port'])
#        return list(set(_ports))
#
#    def remote_addrs(self):
#        '''
#        get dict of remote address and ports list
#        {
#            'remote_host_ip': [list of ports],
#        }
#        '''
#        _addrs = {}
#        for _conn in self.tcp_connexion():
#            if _conn['rem_address']['ip'] not in _addrs:
#                _addrs[_conn['rem_address']['ip']] = []
#            if _conn['rem_address']['port'] not in _addrs[_conn['rem_address']['ip']]:
#                _addrs[_conn['rem_address']['ip']].append(_conn['rem_address']['port'])
#        return _addrs
#
#    def local_addrs(self):
#        '''
#        get dict of local address and ports list
#        {
#            'local_host_ip': [list of ports]
#        }
#        '''
#        _addrs = {}
#        for _conn in self.tcp_connexion():
#            if _conn['local_address']['ip'] not in _addrs:
#                _addrs[_conn['local_address']['ip']] = []
#            if _conn['local_address']['port'] not in _addrs[_conn['local_address']['ip']]:
#                _addrs[_conn['local_address']['ip']].append(_conn['local_address']['port'])
#        return _addrs
#
#
#


class version:
    package = '0.1.7 beta'
    InterfacesInfos = '1.2'
    InterfacesRoutes = '1.2'
    IfaceInfoTools = '0.6'
    FileReader = '0.2'
    Conn = '1.0'
    TCPConn = '1.2'
    UDPConn = '1.2'


from setuptools import setup
#from distutils.core import setup

setup(name='ifaceinfo',
      version='0.1.9',
      description='python class that return networks informations (linux OS) in json/dict format.',
      url='https://github.com/atam84/ifaceinfo',
      author='Mohamed Amine TAMDY',
      author_email='amine.tamdy@gmail.com',
      license='MIT',
      packages=['ifaceinfo'],
      keywords='network interface route tcp udp analyser development',
      classifiers=[  # Optional
            # How mature is this project? Common values are
            #   3 - Alpha
            #   4 - Beta
            #   5 - Production/Stable
            'Development Status :: 3 - Alpha',
            # Indicate who your project is intended for
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Build Tools',
            # Pick your license as you wish
            'License :: OSI Approved :: MIT License',
            # Specify the Python versions you support here. In particular, ensure
            # that you indicate whether you support Python 2, Python 3 or both.
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
      ],
)
